package seq

import (
	"errors"
	"fmt"
	"leit.com/leit_seat_aps/common"
	"strconv"
	"strings"
	"time"
)

// SEQ 数据交换头
type UNB struct {
	Sender      string    // 发送方
	Reciever    string    // 接收方
	MsgTime     time.Time // 消息时间
	MsgCtrlPara string    // 消息交互控制参数
	ParseString string    // 原始消息数据串
}

// SEQ 数据消息头
type UNH struct {
	MsgId          string // 消息标识ID
	MsgType        string // 消息类型 DELJIT
	MsgTypeVersion string // 消息类型版本号
	MsgTypeRelNo   string // 消息类型版本释放号
	ParseString    string // 原始数据串
}

// SEQ 数据消息开始
type BGM struct {
	MessageType int    // 消息类型 30 = TOD / 221 = SEQ / 36 = CALLOFF
	MessageNum  string // 消息编号
	ParseString string // 原始数据串
}

// SEQ 数据消息日期时间信息
type DTM struct {
	DateTimeType   int       // 137 = 消息日期 / 42 = 计划开始装配时间 / 2 = SWET 客户要求的来料时间
	DateTimeFormat int       // 日期时间格式：203 = CCYYMMDDHHMM / 204 = CCYYMMDDHHMMSS
	MsgTime        time.Time // 消息时间
	MsgTimeFormat  string    // 消息时间格式
	ParseString    string    // 原始数据串
}

// FTX 文本区
type FTX struct {
	PartFamily  string // 零件族
	ParseString string // 原始数据串
}

// NAD 数据消息名称地址信息
type NAD struct {
	Consignee        string // 收货人
	ConsigneePlant   int    // 收货厂编号 10=Dadong，66=Shengbei；88=Tiexi
	ConsigneeCompany int    // 收货公司编号 92 = BBA
	ParseString      string // 原始数据串
}

// SEQ 排序号
type SEQ struct {
	SeqType     int    // 3=新建；1=变更；2=取消
	SeqNum      string // 10位排序号
	Dtm         DTM    // 时间对象
	Gir         GIR
	Loc44       LOC    // 文档发出地信息
	Loc83       LOC    // 货物发运地信息
	ParseString string // 原始数据串
}

// SEQ 排序订单相关信息
type GIR struct {
	ReleaseNum    string //
	CheckSequence string // 检查顺序号
	CustOrderNr   string // 客户订单号
	Vin           string // 车身号
	ParseString   string // 原始数据串
}

// SEQ 地点信息
// First 4 characters “M010” is the assembly line code. Supplier’s system
// must use the assembly line code and plant site code in NAD segment
// as group key to get the correct BBA assembly line location.
type LOC struct {
	LocType        int    // 44 = 文档发出地 83 = 货物发运地
	AssemblyLine   string // 排序装配线
	UnloadingPoint string // 使用地点
	ParseString    string // 原始数据串
}

// SEQ 数据消息尾
type UNT struct {
	DataRows    int    // 数据行数，从UNH到UNT总数
	MsgId       string // 消息标识
	ParseString string // 原始数据串
}

// SEQ 数据交换结束符
type UNZ struct {
	MsgCount    int    // 消息数
	MsgRefId    string // 消息参考号
	ParseString string // 原始数据串
}

// SEQUENCE 消息对象
type SEQUENCE struct {
	FileName string // TOD文件名
	DataRows int    // 包含数据行 UNH->UNT
	Unb      UNB    // 数据交换头
	Unh      UNH    // 数据消息头
	Bgm      BGM    // 数据消息开始
	Dtm      DTM    // 数据消息日期时间信息
	Ftx      FTX    // 文本信息
	Nad      NAD    // 数据消息名称地址信息
	SeqList  []SEQ  // 数据消息的零件族信息：零件、零件族、供应组、数量等信息
	Unt      UNT    // 数据消息尾
	Unz      UNZ    // 数据消息交换结束
}

func (unb *UNB) Parse(parseString string) (err error) {
	var (
		kvlist []string
		kv4    []string
	)
	// 输入为空，返回错误
	if strings.TrimSpace(parseString) == "" {
		return errors.New("UNB: parse string is empty!")
	}
	// 基于+分隔符来解析字符串
	kvlist = strings.Split(parseString, "+")
	if len(kvlist) < 5 {
		return errors.New("UNB: Key value length less than 5, failed to parse!")
	}
	kv4 = strings.Split(kvlist[4], ":")
	if len(kv4) < 2 {
		return errors.New("UNB: Key4 value length less than 2, failed to parse!")
	}

	// 填充值
	unb.Sender = kvlist[2]   // 发送方
	unb.Reciever = kvlist[3] // 接收方
	if unb.MsgTime, err = common.DateParse(kv4[0]+kv4[1], "ymdHi"); err != nil {
		return errors.New("UNB: Failed to get the MsgTime!")
	}
	unb.MsgCtrlPara = kvlist[5] // 消息控制参数
	unb.ParseString = parseString

	// 校验关键字段是否为空
	if strings.TrimSpace(unb.Sender) == "" {
		return errors.New("UNB: Sender is empty!")
	}
	if strings.TrimSpace(unb.Reciever) == "" {
		return errors.New("UNB: Reciever is empty!")
	}
	return
}

func (unb *UNB) Print() {
	fmt.Println("UNB | Sender:", unb.Sender, " | Reciever:", unb.Reciever, " | MsgTime:", unb.MsgTime, " | MsgCtrlPara:", unb.MsgCtrlPara)
}

func (unh *UNH) Parse(parseString string) (err error) {
	var (
		kvlist []string
		kv2    []string
	)
	// 输入为空，返回错误
	if strings.TrimSpace(parseString) == "" {
		return errors.New("UNH: parse string is empty!")
	}
	// 基于+分隔符来解析字符串
	kvlist = strings.Split(parseString, "+")
	kv2 = strings.Split(kvlist[2], ":")
	if len(kv2) < 3 {
		return errors.New("UNH: kv2 value length less than 3, failed to parse!")
	}

	// 填充值
	unh.MsgId = kvlist[1] // 消息ID
	unh.MsgType = kv2[0]  // 消息类型
	unh.MsgTypeVersion = kv2[1]
	unh.MsgTypeRelNo = kv2[2]
	unh.ParseString = parseString

	// 校验关键字段是否为空
	if strings.TrimSpace(unh.MsgId) == "" {
		return errors.New("UNH: MsgId is empty!")
	}
	if strings.TrimSpace(unh.MsgType) == "" {
		return errors.New("UNH: MsgType is empty!")
	}
	if strings.ToUpper(strings.TrimSpace(unh.MsgType)) != "DELJIT" {
		return errors.New("UNH: MsgType is not DELJIT!")
	}
	return
}

func (unh *UNH) Print() {
	fmt.Println("UNH | MsgId:", unh.MsgId, " | MsgType:", unh.MsgType)
}

func (bgm *BGM) Parse(parseString string) (err error) {
	var (
		kvlist []string
		kv1    []string
	)
	// 输入为空，返回错误
	if strings.TrimSpace(parseString) == "" {
		return errors.New("BGM: parse string is empty!")
	}
	// 基于+分隔符来解析字符串
	kvlist = strings.Split(parseString, "+")
	kv1 = strings.Split(kvlist[1], ":")
	if len(kv1) < 3 {
		return errors.New("BGM: kv1 value length less than 3, failed to parse!")
	}

	// 填充值
	bgm.MessageType, _ = strconv.Atoi(kv1[0]) // 30=TOD；242=SEQ
	bgm.MessageNum = kvlist[2]                // 消息号
	bgm.ParseString = parseString

	// 校验关键字段是否为空
	if strings.TrimSpace(bgm.MessageNum) == "" {
		return errors.New("BGM: DailyPackageId is empty!")
	}
	return
}

func (bgm *BGM) Print() {
	fmt.Println("BGM | MessageNum:", bgm.MessageNum)
}

// SEQ 之前的：DTM+137:200810021211:203'
// SEQ 之后的：DTM+42:200810092053:203'
func (dtm *DTM) Parse(parseString string) (err error) {
	var (
		kvlist []string
		kv1    []string
	)
	// 输入为空，返回错误
	if strings.TrimSpace(parseString) == "" {
		return errors.New("DTM: parse string is empty!")
	}
	// 基于+分隔符来解析字符串
	kvlist = strings.Split(parseString, "+")
	kv1 = strings.Split(kvlist[1], ":")
	if len(kv1) < 3 {
		return errors.New("DTM: kv1 value length less than 3, failed to parse!")
	}

	// 填充值
	dtm.DateTimeType, _ = strconv.Atoi(kv1[0]) // 时间格式
	dtm.DateTimeFormat, _ = strconv.Atoi(kv1[2])
	dtm.MsgTimeFormat = common.GetTimeFormatByType(dtm.DateTimeFormat)
	if dtm.MsgTime, err = common.DateParse(kv1[1], dtm.MsgTimeFormat); err != nil {
		return errors.New("DTM: Failed to get the MsgTime!")
	}
	dtm.ParseString = parseString

	// 校验关键字段是否为空
	if strings.TrimSpace(dtm.MsgTimeFormat) == "" {
		return errors.New("DTM: MsgTimeFormat is empty!")
	}
	return
}

func (dtm *DTM) Print() {
	fmt.Println("DTM | DateTimeType:", dtm.DateTimeType, " | DateTimeFormat:", dtm.DateTimeFormat, " | MsgTime:", dtm.MsgTime)
}

func (ftx *FTX) Parse(parseString string) (err error) {
	var (
		kvlist []string
		kv3    []string
	)
	// 输入为空，返回错误
	if strings.TrimSpace(parseString) == "" {
		return errors.New("FTX: parse string is empty!")
	}
	// 基于+分隔符来解析字符串
	kvlist = strings.Split(parseString, "+")
	kv3 = strings.Split(kvlist[3], ":")
	if len(kv3) < 3 {
		return errors.New("FTX: kv3 value length less than 3, failed to parse!")
	}

	// 填充值
	ftx.PartFamily = kvlist[4] // 零件族
	ftx.ParseString = parseString

	// 校验关键字段是否为空
	if strings.TrimSpace(ftx.PartFamily) == "" {
		return errors.New("FTX: Partfamily is empty!")
	}
	return
}

func (ftx *FTX) Print() {
	fmt.Println("FTX | Partfamily:", ftx.PartFamily)
}

func (nad *NAD) Parse(parseString string) (err error) {
	var (
		kvlist []string
		kv2    []string
	)
	// 输入为空，返回错误
	if strings.TrimSpace(parseString) == "" {
		return errors.New("NAD: parse string is empty!")
	}
	// 基于+分隔符来解析字符串
	kvlist = strings.Split(parseString, "+")
	kv2 = strings.Split(kvlist[2], ":")
	if len(kv2) < 1 {
		return errors.New("NAD: kv2 value length less than 1, failed to parse!")
	}

	// 填充值
	nad.Consignee = kvlist[1]                      // 收货人
	nad.ConsigneePlant, _ = strconv.Atoi(kv2[0])   // 收货厂编号 10=Dadong，66=Shengbei；88=Tiexi
	nad.ConsigneeCompany, _ = strconv.Atoi(kv2[2]) // 收货公司编号 92 = BBA
	nad.ParseString = parseString

	// 校验关键字段是否为空
	if strings.TrimSpace(nad.Consignee) == "" {
		return errors.New("NAD: Consignee is empty!")
	}
	if nad.ConsigneePlant <= 0 {
		return errors.New("NAD: ConsigneePlant is not existing!")
	}
	if nad.ConsigneeCompany <= 0 {
		return errors.New("NAD: ConsigneeCompany is not existing!")
	}
	return
}

func (nad *NAD) Print() {
	fmt.Println("NAD | Consignee:", nad.Consignee, " | ConsigneePlant:", nad.ConsigneePlant, " | ConsigneeCompany:", nad.ConsigneeCompany)
}

func (seq *SEQ) Parse(parseString string) (err error) {
	var (
		kvlist []string
	)
	// 输入为空，返回错误
	if strings.TrimSpace(parseString) == "" {
		return errors.New("SEQ: parse string is empty!")
	}
	// 基于+分隔符来解析字符串
	kvlist = strings.Split(parseString, "+")

	// 填充值
	seq.SeqType, _ = strconv.Atoi(kvlist[1]) // 3=新建；1=更新；2=取消
	if len(kvlist) > 2 {
		seq.SeqNum = kvlist[2] // 排序号
	}
	seq.ParseString = parseString

	// 校验关键字段是否为空
	if seq.SeqType < 1 || seq.SeqType > 3 {
		return errors.New("SEQ: Unvalid seq type!")
	}
	return
}

func (seq *SEQ) Print() {
	fmt.Println("SEQ | SEQ + ", seq.SeqType, " | SeqNum:", seq.SeqNum)
}

// 示例：GIR+1+08483631:AN+SW72831:VN
func (gir *GIR) Parse(parseString string) (err error) {
	var (
		kvlist   []string
		kv2, kv3 []string
	)
	// 输入为空，返回错误
	if strings.TrimSpace(parseString) == "" {
		return errors.New("GIR: parse string is empty!")
	}
	// 基于+分隔符来解析字符串
	// GIR+1+00024391:AQ+00000966:ML+12373531:AN + GX38594 : VN
	kvlist = strings.Split(parseString, "+")
	kv2 = strings.Split(kvlist[2], ":")
	kv3 = strings.Split(kvlist[3], ":")
	if len(kv2) < 1 || len(kv3) < 1 {
		return errors.New("GIR: kv2/3/4/5 value length less than 1, failed to parse!")
	}

	// 填充值
	gir.CustOrderNr = kv2[0] // 客户订单号
	gir.Vin = kv3[0]         // 车身号
	gir.ParseString = parseString

	// 校验关键字段是否为空
	if strings.TrimSpace(gir.CustOrderNr) == "" {
		return errors.New("GIR: CustOrderNr is empty!")
	}
	if strings.TrimSpace(gir.Vin) == "" {
		return errors.New("GIR: Vin is empty!")
	}
	return
}

func (gir *GIR) Print() {
	fmt.Println("GIR | CustOrderNr:", gir.CustOrderNr, " | Vin:", gir.Vin)
}

// 示例1：LOC+44+M01072
// 示例2：LOC+83+0000M0100001
func (loc *LOC) Parse(parseString string) (err error) {
	var (
		kvlist []string
	)
	// 输入为空，返回错误
	if strings.TrimSpace(parseString) == "" {
		return errors.New("LOC: parse string is empty!")
	}
	// 基于+分隔符来解析字符串
	kvlist = strings.Split(parseString, "+")

	// 填充值
	loc.LocType, _ = strconv.Atoi(kvlist[1]) // 44=文档发出地；83=发运目的地
	switch loc.LocType {
	case 44:
		loc.AssemblyLine = kvlist[2]
	case 83:
		loc.UnloadingPoint = kvlist[2]
	}
	loc.ParseString = parseString

	// 校验关键字段是否为空
	if loc.LocType <= 0 {
		return errors.New("LOC: LocType is empty!")
	}
	return
}

func (loc *LOC) Print() {
	fmt.Println("LOC | LocType:", loc.LocType, " | AssemblyLine:", loc.AssemblyLine, " | UnloadingPoint:", loc.UnloadingPoint)
}

func (unt *UNT) Parse(parseString string) (err error) {
	var (
		kvlist []string
	)
	// 输入为空，返回错误
	if strings.TrimSpace(parseString) == "" {
		return errors.New("UNT: parse string is empty!")
	}
	// 基于+分隔符来解析字符串
	kvlist = strings.Split(parseString, "+")

	// 填充值
	if unt.DataRows, err = strconv.Atoi(kvlist[1]); err != nil {
		return errors.New("UNT: Failed to get data rows!")
	}
	unt.MsgId = kvlist[2]
	unt.ParseString = parseString

	// 校验关键字段是否为空
	if strings.TrimSpace(unt.MsgId) == "" {
		return errors.New("UNT: MsgId is empty!")
	}
	return
}

func (unt *UNT) Print() {
	fmt.Println("UNT | MsgId:", unt.MsgId, " | DataRows:", unt.DataRows)
}

func (unz *UNZ) Parse(parseString string) (err error) {
	var (
		kvlist []string
	)
	// 输入为空，返回错误
	if strings.TrimSpace(parseString) == "" {
		return errors.New("UNZ: parse string is empty!")
	}
	// 基于+分隔符来解析字符串
	kvlist = strings.Split(parseString, "+")

	// 填充值
	if unz.MsgCount, err = strconv.Atoi(kvlist[1]); err != nil {
		return errors.New("UNZ: Failed to get message count!")
	}
	unz.MsgRefId = kvlist[2]
	unz.ParseString = parseString

	// 校验关键字段是否为空
	if strings.TrimSpace(unz.MsgRefId) == "" {
		return errors.New("UNZ: MsgRefId is empty!")
	}
	return
}

func (unz *UNZ) Print() {
	fmt.Println("UNZ | MsgRefId:", unz.MsgRefId, " | MsgCount:", unz.MsgCount)
}
